"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractAmqpPublisher = void 0;
const node_core_1 = require("@lokalise/node-core");
const core_1 = require("@message-queue-toolkit/core");
const AbstractAmqpService_1 = require("./AbstractAmqpService");
class AbstractAmqpPublisher extends AbstractAmqpService_1.AbstractAmqpService {
    messageSchemaContainer;
    constructor(dependencies, options) {
        super(dependencies, options);
        const messageSchemas = options.messageSchemas;
        this.messageSchemaContainer = new core_1.MessageSchemaContainer({
            messageSchemas,
            messageTypeField: options.messageTypeField,
        });
    }
    publish(message) {
        const resolveSchemaResult = this.resolveSchema(message);
        if (resolveSchemaResult.error) {
            throw resolveSchemaResult.error;
        }
        resolveSchemaResult.result.parse(message);
        /**
         * If the message doesn't have a timestamp field -> add it
         * will be used on the consumer to prevent infinite retries on the same message
         */
        if (!this.tryToExtractTimestamp(message)) {
            // @ts-ignore
            message[this.messageTimestampField] = new Date().toISOString();
            this.logger.warn(`${this.messageTimestampField} not defined, adding it automatically`);
        }
        if (this.logMessages) {
            // @ts-ignore
            const resolvedLogMessage = this.resolveMessageLog(message, message[this.messageTypeField]);
            this.logMessage(resolvedLogMessage);
        }
        try {
            this.channel.sendToQueue(this.queueName, (0, core_1.objectToBuffer)(message));
        }
        catch (err) {
            // Unfortunately, reliable retry mechanism can't be implemented with try-catch block,
            // as not all failures end up here. If connection is closed programmatically, it works fine,
            // but if server closes connection unexpectedly (e. g. RabbitMQ is shut down), then we don't land here
            // @ts-ignore
            if (err.message === 'Channel closed') {
                this.logger.error(`AMQP channel closed`);
                void this.reconnect();
            }
            else {
                throw new node_core_1.InternalError({
                    message: `Error while publishing to AMQP ${err.message}`,
                    errorCode: 'AMQP_PUBLISH_ERROR',
                    details: {
                        publisher: this.constructor.name,
                        queueName: this.queueName,
                        // @ts-ignore
                        messageType: message[this.messageTypeField] ?? 'unknown',
                    },
                    cause: err,
                });
            }
        }
    }
    resolveSchema(message) {
        return this.messageSchemaContainer.resolveSchema(message);
    }
    /* c8 ignore start */
    resolveMessage() {
        throw new Error('Not implemented for publisher');
    }
    /* c8 ignore start */
    processPrehandlers() {
        throw new Error('Not implemented for publisher');
    }
    preHandlerBarrier() {
        throw new Error('Not implemented for publisher');
    }
    resolveNextFunction() {
        throw new Error('Not implemented for publisher');
    }
    processMessage() {
        throw new Error('Not implemented for publisher');
    }
}
exports.AbstractAmqpPublisher = AbstractAmqpPublisher;
//# sourceMappingURL=AbstractAmqpPublisher.js.map